/*
 *                   jfjji,.           H A M M E R H E A D  S H A R K W A R E
 *                  ,#######EL:                  http://www.hhsw.de
 *                   .,j######Kj       
 *                    ;W########G:     This  code  is  free software; you can
 *                   j#####G ,D#L:     redistribute it and/or modify it under
 *             .,;;,G#####D.           the  terms  of  the GNU General Public
 *           jE##########G             License   as  published  by  the  Free
 *             .,W######D.             Software  Foundation; either version 2
 *             :L#####K##G             of  the  License,  or (at your option)
 *           .f#####G;.t##:            any later version.
 *         ;L###ELi.    ,D,            
 *       iEEGWj                        This  code  is distributed in the hope
 *      tW;                            that  it  will  be useful, but without
 *     jf                              any warranty; without even the implied
 *    .Kt                              warranty of merchantability or fitness
 *    i:                               for  a particular purpose. See the GNU
 *    ;                                General   Public   License   for  more
 *    .                                details (http://www.gnu.org).
 */
#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <iostream>
#include <fstream>

#include "Dock.hh"

using namespace std;
using namespace bbdock;

/**
 * Add an Icon record to a Dock
 *
 * @param dock - Dock object
 * @param record - an icon record
 * @param seperator - field seperators
 */
bool add( Dock &dock, char *record, const char *seperator )
{
	// cut off comments
	{
		char	*comment;

		if( (comment = strchr( record, '#' )) )
			*comment = 0;
	}

	char	*img;
	char	*cmd;

	if( !(img = strtok( record, seperator )) ||
		!(cmd = strtok( 0, seperator )) )
		return false;

	char	*title = strtok( 0, seperator );

	return dock.add( new Icon( img, cmd, title ) );
}

/**
 * Load icons from file
 *
 * @param binary - name of the currently running binary
 * @param dock - Dock object
 */
void loadRC( char *binary, Dock &dock )
{
	string				rcfilename = binary;
	string::size_type	p;

	if( (p = rcfilename.find_last_of( '/' )) )
		rcfilename = rcfilename.substr( ++p );

	rcfilename = "/."+rcfilename+"rc";
	rcfilename = getenv( "HOME" )+rcfilename;

	ifstream	fin( rcfilename.c_str(), ios::in );

	if( !fin )
		throw "Unable to open resource configuraton file for reading";

	for( string buf; getline( fin, buf ); )
		add( dock, (char *) buf.c_str(), ":" );
}

/**
 * Entry point
 */
int main( int argc, char **argv )
{
	int		slotwidth = 64;
	int		slotheight = 64;
	char 	*binary;

	// process command line arguments  
	for( binary = *argv; --argc; )
		if( **(++argv) != '-' )
			break;
		else
			switch( *((*argv)+1) )
			{
				default:
					cerr << "Unknown argument \"" << *argv << "\" !" << endl 
						<< endl;
					// fall through
				case '?':
				case 'h':
					cout << "usage: " << binary << 
						" [-hvdi] PNGFILE:COMMAND[:WINDOWTITLE]..." << endl <<
						"  -h               print this help" << endl <<
						"  -v               print version" << endl <<
						"  -d WIDTHxHEIGHT  dimensions of dock buttons" << 
						endl <<
						"  -i COMMAND:ICON  change image of an icon" << 
						endl << endl <<
						"PNGFILE     - some image file in PNG format" << 
						endl <<
						"COMMAND     - a script or binary to execute" <<
						endl <<
						"WINDOWTITLE - a (case-insensitive) part of the " <<
						"window-title of an exclusive" << endl <<
						"              application; this icon will watch " <<
						"a running instance and any" << endl <<
						"              further click will raise the " <<
						"corresponding window" << 
						endl;
					return 0;
				case 'v':
					cout << "bbdock, version 0.08 <mf@hhsw.de>" << endl;
					return 0;
				case 'd':
					{
						char	*w;
						char	*h;

						if( --argc &&
							(w = strtok( *(++argv), "x" )) &&
							(h = strtok( 0, "" )) )
						{
							slotwidth = atoi( w );
							slotheight = atoi( h );
						}
						else
							cerr << "Missing or invalid dimensions !" << 
								endl;
					}
					break;
				case 'i':
					{
						char	*icon;
						char	*cmd;

						if( --argc &&
							(cmd = strtok( *(++argv), ":" )) &&
							(icon = strtok( 0, "" )) )
						{
							Dock::changeIcon( cmd, icon );
							return 0;
						}
						else
							cerr << "Missing or invalid parameter !" << 
								endl;
					}
					break;
			}

	// detach from shell
	switch( fork() )
	{
		case -1:
			cerr << "Can not fork !" << endl;
			return -1;
		case 0:
			break;
		default:
			return 0;
	}

	// bring up the icons
	try
	{
		Dock	dock( slotwidth, slotheight );

		if( !argc )
		{
			// try to start up by configuration file
			loadRC( binary, dock );
		}
		else
		{
			// try to start up by command line
			for( ; argc--; argv++ )
				if( !add( dock, *argv, ":" ) )
					throw "Invalid argument";
		}
		
		dock.run();
	}
	catch( const char *e )
	{
		cerr << e << endl;
	}
	catch( ... )
	{
		cerr << "Unknown error" << endl;
	}

	return 0;
}
